/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  Copyright (c) 2003-2004                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <cstdlib>
#include <iostream>
#include <iomanip>

#include "earl.h"

using namespace earl;
using namespace std;

ostream& operator<<(ostream& out, const Location& loc)
{
  //out.width(13);
  out << "LOCATION " << "[" 
      << loc.get_id() << "] ( "
      << loc.get_mach()->get_id() << ", "
      << loc.get_node()->get_node_id() << ", "
      << loc.get_proc()->get_id() << ", " 
      << loc.get_thrd()->get_thrd_id() << " )\n"; 
  return out;
}

ostream& operator<<(ostream& out, const Machine& mach)
{
  //out.width(13);
  out << "MACHINE " << "[" 
      << mach.get_id() << "] ( "
      << mach.get_name() << ", #nodes: " 
      << mach.get_nnodes() << " )\n";
  return out;
}

ostream& operator<<(ostream& out, const Node& node)
{
  //out.width(13);
  out << "NODE " << "[" 
      << node.get_mach_id() << ", "
      << node.get_node_id() << "] ( "
      << node.get_name() << ", #cpus: " 
      << node.get_ncpus() << ", clock rate: "
      << node.get_clckrt() << ", #procs: "
      << node.get_nprocs() << " )\n";
  return out;
}

ostream& operator<<(ostream& out, const Process& proc)
{
  //out.width(13);
  out << "PROCESS " << "[" 
      << proc.get_id() << "] ( "
      << proc.get_name() << ", #thrds: " 
      << proc.get_nthrds() << " )\n";
  return out;
}

ostream& operator<<(ostream& out, const Thread& thrd)
{
  //out.width(13);
  out << "THREAD " << "[" 
      << thrd.get_proc_id() << ", "
      << thrd.get_thrd_id() << "] ( "
      << thrd.get_name() << " )\n";
  return out;
}

ostream& operator<<(ostream& out, const Callsite& csite)
{
  out << "CALLSITE " << "[" 
      << csite.get_id() << "] ( "
      << csite.get_file() << ":" 
      << csite.get_line() << " -> "
      << csite.get_callee()->get_name() << " )\n";
  return out;
}

ostream& operator<<(ostream& out, const Region& reg)
{
  //out.width(13);
  out << "REGION " << "[" 
      << reg.get_id() << "] ( "
      << reg.get_name() << ", " 
      << reg.get_file() << ":" 
      << reg.get_begln() << "/"
      << reg.get_endln() << ", "
      << reg.get_rtype() << ", "
      << reg.get_descr() << " )\n";
  return out;
}

ostream& operator<<(ostream& out, const Metric& met)
{
  //out.width(13);
  out << "METRIC " << "[" 
      << met.get_id()  << "] ( "
      << met.get_name() << ", "
      << met.get_descr() << ", "
      << met.get_type() << ", "
      << met.get_mode() << ", "     
      << met.get_ival() << " )\n";
  return out;
}


ostream& operator<<(ostream& out, const Communicator& com)
{
  //out.width(13);
  out << "COMMUNICATOR " << "[" 
      << com.get_id()  << "] ( ";   
  for ( int i = 0; i < com.size(); i++)
    out << com.get_proc(i)->get_id() << " ";
  out << " )\n";
  return out;
}

ostream& operator<<(ostream& out, const Event& event)
{
  out << setprecision(6);

  //out.width(14);
  out << event.get_typestr() << "[" 
      << event.get_pos() << "] (" 
      << event.get_loc()->get_mach()->get_id() << ", "
      << event.get_loc()->get_node()->get_node_id() << ", "
      << event.get_loc()->get_proc()->get_id() << ", "
      << event.get_loc()->get_thrd()->get_thrd_id() << ") "
      << event.get_time() << "\n";
  if ( !event.get_enterptr().null() )
    out << " enterptr       " << event.get_enterptr().get_pos() << "\n";
  if ( !event.get_cnodeptr().null() )
    out << " cnodeptr       " << event.get_cnodeptr().get_pos() << "\n";
  if ( !event.get_cedgeptr().null() )
    out << " cedgeptr       " << event.get_cedgeptr().get_pos() << "\n";
  if ( event.get_reg() != NULL )
    out << " region         " << event.get_reg()->get_name() << "\n";
  if ( event.get_csite() != NULL )
    out << " callsite       " << event.get_csite()->get_callee()->get_name() << "\n";
  if ( event.get_type() == SEND || event.get_type() == RECV || event.get_type() == MPICEXIT )
      out << " communicator   " << event.get_com()->get_id() << "\n";    
  if ( event.get_type() == SEND || event.get_type() == RECV )
    {
      out << " length         " << event.get_length() << "\n";
      out << " tag            " << event.get_tag() << "\n";
    }
  if ( event.get_src() != NULL ) 
      out << " src          " << event.get_src()->get_id() << "\n";
  if ( event.get_dest() != NULL ) 
      out << " dest         " << event.get_dest()->get_id() << "\n";
  if ( !event.get_sendptr().null() )
    out << " sendptr        " << event.get_sendptr().get_pos() << "\n";
  if ( event.get_root() != NULL ) 
      out << " root         " << event.get_root()->get_id() << "\n";
  if ( event.get_type() == MPICEXIT )
    {
      out << " sent            " << event.get_sent() << "\n";
      out << " recvd           " << event.get_recvd() << "\n";
    }
  if ( !event.get_forkptr().null() )
    out << " forkptr        " << event.get_forkptr().get_pos() << "\n";
  if ( !event.get_lockptr().null() )
    out << " lockptr        " << event.get_lockptr().get_pos() << "\n";
  if ( event.get_type() == ALOCK || event.get_type() == RLOCK )
    out << " lock_id        " << event.get_lock_id() << "\n";
  for ( int i = 0; i < event.get_nmets(); i++ )
    out << event.get_metname(i) << " " << event.get_metval(i) << "\n";
  return out;
}  
 
ostream& operator<<(ostream& out, const vector<Event>& eventv)
{
  for ( int i = 0; i < eventv.size(); i++ )
    out << " " << eventv[i].get_pos();
  out << "\n";
  return out;
}
    

void print_enterv(const vector<Event>& eventv)
{
  for ( int i = 0; i < eventv.size(); i++ )
    {
      cout << "(" << eventv[i].get_reg()->get_name() << ", ";
      if ( eventv[i].get_csite() )
	cout << eventv[i].get_csite()->get_line();
      else
	cout << "-";
      cout << ") ";
    }
  cout << "\n";
}

void print_sendv(const vector<Event>& eventv)
{
  for ( int i = 0; i < eventv.size(); i++ )
    cout << "(" << eventv[i].get_loc()->get_id() << ", " << eventv[i].get_dest()->get_id() << ") ";
  cout << "\n";
}

void print_exitv(const vector<Event>& eventv)
{
  for ( int i = 0; i < eventv.size(); i++ )
    cout << "(" << eventv[i].get_reg()->get_name() << ") ";
  cout << "\n";
}

void test_calltree(EventTrace& trace)
{
  cout << endl;  
  cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
  cout << "+ CALLTREE " << endl;
  cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;

  vector<Event> ctv;
    
  trace.calltree(ctv);
  
  for ( int i = 0; i < ctv.size(); i++ )
    {
      vector<Event> cpv;
      trace.callpath(cpv, ctv[i].get_pos());

      for ( int j = 0; j < cpv.size(); j++ )
	{
	  string region;
	  long line;
	  
	  region = cpv[j].get_reg()->get_name();
	  if ( cpv[j].get_csite() != NULL )
	    line = cpv[j].get_csite()->get_line();
	  else
	    line = -1;
	  
	  cout << region << ":" << line << " ";
	}
      cout << endl;
    }
}

void test_static_data(EventTrace& trace)
{
  cout << endl;  
  cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
  cout << "+ STATIC DATA " << endl;
  cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
  
  for ( int i = 0; i < trace.get_nmachs(); i++ )
    {
      cout << *trace.get_mach(i);
      for ( int j = 0; j < trace.get_mach(i)->get_nnodes(); j++ )
	cout << *trace.get_node(i,j);
    }
  for ( int i = 0; i < trace.get_nprocs(); i++ )
    {
      cout << *trace.get_proc(i);
      for ( int j = 0; j < trace.get_proc(i)->get_nthrds(); j++ )
	cout << *trace.get_thrd(i,j);
    }
  for ( int i = 0; i < trace.get_nlocs(); i++ )
    cout << *trace.get_loc(i);
  for ( int i = 0; i < trace.get_nfiles(); i++ )
    {
      cout << "FILE " << "[" << i << "] ( " << trace.get_file(i) << " )\n";
    }
  for ( int i = 0; i < trace.get_nregs(); i++ )
    cout << *trace.get_reg(i);
    for ( int i = 0; i < trace.get_ncsites(); i++ )
      cout << *trace.get_csite(i);
    for ( int i = 0; i < trace.get_nmets(); i++ )
      cout << *trace.get_met(i);
    for ( int i = 0; i < trace.get_ncoms(); i++ )
      cout << *trace.get_com(i);
}

void test_dynamic_data(EventTrace& trace)
{
  cout << endl;  
  cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
  cout << "+ DYNAMIC DATA " << endl;
  cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
  

  for ( int i = 1; i <= trace.get_nevents(); i++ )
      {
	Event event = trace.event(i);
	cout << event << "\n";
	
	vector<Event> ev;
	if (  event.is_type(FLOW) )
	  {
	    cout << "STACK ";
	    trace.stack(ev, i, event.get_loc()->get_id()); 
	    print_enterv(ev); 
	  }
	if (  event.is_type(ENTER) )
	  {
	    cout << "CTREE ";
	    trace.calltree(ev, i);
	    print_enterv(ev);
	    cout << "CPATH ";
	    trace.callpath(ev, event.get_cnodeptr().get_pos());
	    print_enterv(ev);
	  }
	if (  event.is_type(MPICEXIT) )
	  {
	    cout << "MPICOLL ";
	    trace.mpicoll(ev, i);
	    print_exitv(ev);
	  }
	if (  event.is_type(OMPCEXIT) )
	  {
	    cout << "OMPCOLL ";
	    trace.ompcoll(ev, i);
	    print_exitv(ev);
	  }
	if (  event.is_type(SEND) )
	  {
	    cout << "QUEUE ";
	    trace.queue(ev, i, event.get_loc()->get_id(), event.get_dest()->get_id());
	    print_sendv(ev);
	  }
	if (  event.is_type(RECV) )
	  {
	    cout << "QUEUE ";
	    trace.queue(ev, i, event.get_src()->get_id(), event.get_loc()->get_id());
	    print_sendv(ev);
	  }
	cout << "\n";
      }
  for ( int i = trace.get_nevents(); i > 0; i-- )
      {
	Event event = trace.event(i);
	cout << event << "\n";
	
	vector<Event> ev;
	if (  event.is_type(FLOW) )
	  {
	    cout << "STACK ";
	    trace.stack(ev, i, event.get_loc()->get_id()); 
	    print_enterv(ev); 
	  }
	if (  event.is_type(ENTER) )
	  {
	    cout << "CTREE ";
	    trace.calltree(ev, i);
	    print_enterv(ev);
	    cout << "CPATH ";
	    trace.callpath(ev, event.get_cnodeptr().get_pos());
	    print_enterv(ev);
	  }
	if (  event.is_type(MPICEXIT) )
	  {
	    cout << "MPICOLL ";
	    trace.mpicoll(ev, i);
	    print_exitv(ev);
	  }
	if (  event.is_type(OMPCEXIT) )
	  {
	    cout << "OMPCOLL ";
	    trace.ompcoll(ev, i);
	    print_exitv(ev);
	  }
	if (  event.is_type(SEND) )
	  {
	    cout << "QUEUE ";
	    trace.queue(ev, i, event.get_loc()->get_id(), event.get_dest()->get_id());
	    print_sendv(ev);
	  }
	if (  event.is_type(RECV) )
	  {
	    cout << "QUEUE ";
	    trace.queue(ev, i, event.get_src()->get_id(), event.get_loc()->get_id());
	    print_sendv(ev);
	  }
	cout << "\n";
      }
}

int main(int argc, char* argv[])
{
  if ( argc < 2 )
    {
      cerr << "Wrong # arguments.\n";
      exit(EXIT_FAILURE);
    }

  try {

    // open trace file

    EventTrace trace(argv[1]);

    test_calltree(trace);
    test_static_data(trace);
    test_dynamic_data(trace);
  }
  catch ( Error error ) {

    cerr << error.get_msg() << "\n";
    exit(EXIT_FAILURE);
  } 
} 


