!****************************************************************************
!*  KOJAK                                                                  **
!*  http://www.fz-juelich.de/zam/kojak/                                    **
!****************************************************************************
!*  Copyright (c) 1998-2004                                                **
!*  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
!*                                                                         **
!*  See the file COPYRIGHT in the package base directory for details       **
!***************************************************************************/
  
subroutine mpi_cart_topol(rank, size)
  integer :: rank, size

  integer   dim0, dim1, dim2	
  integer*1 period0, period1, period2	
  integer   coord0, coord1, coord2	
	        
  ! run this program with 8 processes
  if ( size .ne. 8 ) then
    print *,'Invalid number of processes'
    print *,'Run this progranm with 8 processes'
    stop
  endif

  dim0 = 2
  dim1 = 2
  dim2 = 2
        
  period0 = 1
  period1 = 0
  period2 = 1

  ! set the user topology using elg_ftopol.f subroutine 
  call elgf_cart_create(dim0, dim1, dim2, period0, period1, period2)	

   if ( rank .eq. 0 ) then 
    coord0 = 0
    coord1 = 0
    coord2 = 0
    print *,'P:',rank,'My coordinates are', coord0, coord1, coord2      
    print *
    call elgf_cart_coords(coord0, coord1, coord2)
  endif

  if ( rank .eq. 1 ) then 
    coord0 = 1
    coord1 = 0
    coord2 = 0
    print *,'P:',rank,'My coordinates are', coord0, coord1, coord2      
    print *
    call elgf_cart_coords(coord0, coord1, coord2)
  endif

  if ( rank .eq. 2) then 
    coord0 = 0
    coord1 = 1
    coord2 = 0
    print *,'P:',rank,'My coordinates are', coord0, coord1, coord2      
    print *
    call elgf_cart_coords(coord0, coord1, coord2)
  endif	

  if ( rank .eq. 3) then 
    coord0 = 1
    coord1 = 1
    coord2 = 0
    print *,'P:',rank,'My coordinates are', coord0, coord1, coord2      
    print *
    call elgf_cart_coords(coord0, coord1, coord2)
  endif	     

  if ( rank .eq. 4) then 
    coord0 = 0
    coord1 = 0
    coord2 = 1
    print *,'P:',rank,'My coordinates are', coord0, coord1, coord2      
    print *
    call elgf_cart_coords(coord0, coord1, coord2)
  endif	  

  if ( rank .eq. 5) then 
    coord0 = 1
    coord1 = 0
    coord2 = 1
    print *,'P:',rank,'My coordinates are', coord0, coord1, coord2      
    print *
    call elgf_cart_coords(coord0, coord1, coord2)
  endif

  if ( rank .eq. 6) then 
    coord0 = 0
    coord1 = 1
    coord2 = 1
    print *,'P:',rank,'My coordinates are', coord0, coord1, coord2      
    print *
    call elgf_cart_coords(coord0, coord1, coord2)
  endif

  if ( rank .eq. 7) then 
    coord0 = 1
    coord1 = 1
    coord2 = 1
    print *,'P:',rank,'My coordinates are', coord0, coord1, coord2      
    print *
    call elgf_cart_coords(coord0, coord1, coord2)
  endif
end subroutine	



subroutine ompi_cart_topol(rank, size)
  integer :: rank, size
 
  integer :: tid=0, max_threads	

  integer ::  dim0, dim1, dim2	
  integer*1 :: period0, period1, period2	
  integer  :: coord0, coord1, coord2
   	

#if (defined (ELG_OMPI) || defined (ELG_OMP))
  integer :: omp_get_thread_num
  integer :: omp_get_max_threads 	
#endif

  ! run this program with 4 processes
  if ( size .ne. 4 ) then
    print *,'Invalid number of processes'
    print *,'Run this progranm with 4 processes'
    stop
  endif	

#if ( defined(ELG_OMPI) ) 
  max_threads = omp_get_max_threads();
#endif  

  dim0 = 2
  dim1 = 2
  dim2 = max_threads

  period0 = 1
  period1 = 0
  period2 = 1

  ! set the user topology using elg_ftopol.f subroutine 
  call elgf_cart_create(dim0, dim1, dim2, period0, period1, period2)

   if ( rank .eq. 0 ) then 
    coord0 = 0
    coord1 = 0    
  endif

  if ( rank .eq. 1 ) then 
    coord0 = 1
    coord1 = 0    
  endif

  if ( rank .eq. 2) then 
    coord0 = 0
    coord1 = 1
  endif	

  if ( rank .eq. 3) then 
    coord0 = 1
    coord1 = 1    
  endif	     


  !$OMP PARALLEL PRIVATE(tid)
#if ( defined(ELG_OMPI)  )    
  tid = omp_get_thread_num();   
#endif
  coord2 = tid
  print *,'P:',rank, 'T:', tid, 'My coordinates are', coord0, coord1, coord2   
  print *
  call elgf_cart_coords(coord0, coord1, tid)
  !$OMP END PARALLEL

end subroutine


subroutine omp_cart_topol()

  integer   ::  dim0, dim1, dim2	
  integer*1 ::  period0, period1, period2	
  integer   ::  coord0, coord1, coord2
  integer   ::  tid=0, max_threads 

#if (defined (ELG_OMPI) || defined (ELG_OMP))
  integer :: omp_get_thread_num
  integer :: omp_get_max_threads 	
#endif

#if (defined(ELG_OMP) ) 
  max_threads = omp_get_max_threads();
#endif  

  dim0 = max_threads
  dim1 = 0
  dim2 = 0

  period0 = 0
  period1 = 0
  period2 = 0

  ! set the user topology using elg_ftopol.f subroutine 
  call elgf_cart_create(dim0, dim1, dim2, period0, period1, period2)	

  !$OMP PARALLEL PRIVATE(tid)
#if ( defined(ELG_OMP)  )    
  tid = omp_get_thread_num();   
#endif 
  print *,'P:',rank, 'T:', tid, 'My coordinates are', tid, '0',' ', '0'   
  print *
  call elgf_cart_coords(tid, 0, 0)
  !$OMP END PARALLEL

end subroutine
    
program main

#if (defined (ELG_MPI) || defined (ELG_OMPI))
  include 'mpif.h'
#endif
  
  integer :: rank, size, ierr  
        
  !$POMP INST INIT
  !$POMP INST BEGIN(main)
       

#if (defined (ELG_MPI) || defined (ELG_OMPI))
  call MPI_Init(ierr)
  call MPI_Comm_size(MPI_COMM_WORLD, size, ierr)
  call MPI_Comm_rank(MPI_COMM_WORLD, rank, ierr)
#endif
  
 
#if ( defined(ELG_MPI) )
  call mpi_cart_topol(rank, size);
#endif

#if ( defined(ELG_OMPI) )
  call ompi_cart_topol(rank, size);
#endif

#if ( defined(ELG_OMP) )
  call omp_cart_topol();
#endif

#if (defined (ELG_MPI) || defined (ELG_OMPI))  
  call MPI_Finalize(ierr);
#endif

!$POMP INST END(main)
end program main


        
 

 


  
      
