/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 2003-2004                                                ** 
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         ** 
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "dlg_mod.h"
#include "elg_error.h"

#include <string.h>
#include <stdlib.h>

struct dlg_mod
{
  long id;
  char* name;
  long nfuncs;
  long* func_idv;
  long tag;
};

struct dlg_mod*   
dlg_mod_create(char* modspec)
{
  char* name;
  int i;
  char* tmp;

  struct dlg_mod* mod = malloc(sizeof(struct dlg_mod));
  if ( mod == NULL )
    elg_error();

  if ( strcmp(strtok(modspec, " "), "MODULE") != 0 )
    elg_error_msg("Invalid module-specification string");
  
  mod->id = atol(strtok(NULL, " "));

  name = strtok(NULL, " ");
  mod->name = malloc(strlen(name) + 1);
  if ( mod->name == NULL )
    elg_error();
  memcpy(mod->name, name, strlen(name) + 1);
  
  mod->nfuncs = atol(strtok(NULL, " "));
  
  if ( mod->nfuncs > 0 )
    {
      mod->func_idv = calloc(mod->nfuncs, sizeof(long));
      if ( mod->func_idv == NULL )
	elg_error();
    }
  else
    {
      mod->func_idv == NULL;
    }
  
  for ( i = 0; i < mod->nfuncs; i++)
    mod->func_idv[i] = atol(strtok(NULL, " "));

  mod->tag = -1;

  return mod;
}

void              
dlg_mod_delete(struct dlg_mod* mod)
{
  free(mod->name);
  free(mod->func_idv);
  free(mod);
}

long              
dlg_mod_id(struct dlg_mod* mod)
{
  return mod->id;
}

const char*      
dlg_mod_name(struct dlg_mod* mod)
{
  return mod->name;
}

long              
dlg_mod_num_funcs(struct dlg_mod* mod)
{
  return mod->nfuncs;
}

long  
dlg_mod_func_id(struct dlg_mod* mod, long i)
{
  return mod->func_idv[i];
}

long              
dlg_mod_get_tag(struct dlg_mod* mod)
{
  return mod->tag;
}

void              
dlg_mod_set_tag(struct dlg_mod* mod, long tag)
{
  mod->tag = tag;
}

void             
dlg_mod_fprintf(FILE* stream, struct dlg_mod* mod)
{
  int i;
  
  fprintf(stream, "MODULE %d %s %d", mod->id, mod->name, mod->nfuncs);  
  for ( i = 0; i < mod->nfuncs; i++ )
    fprintf(stream, " %d", mod->func_idv[i]);
  fprintf(stream, " tag = %d\n", mod->tag);
}
