/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 2003-2004                                                ** 
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         ** 
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "elg_comp_dpcl.h"

#include "dlg_prog.h"
#include "elg_error.h"
#include "elg_trc.h"

static struct dlg_prog* prog = NULL;
static int main_id;

void 
dlg_register_func(struct dlg_func* func)
{
  /* register module (i.e., file) if not already registered */
  struct dlg_mod* mod  = dlg_prog_mod(prog, dlg_func_mod_id(func));
  
  if ( dlg_mod_get_tag(mod) == -1 )
    dlg_mod_set_tag(mod, elg_def_file(dlg_mod_name(mod)));
  
  dlg_func_set_tag(func, elg_def_region(dlg_func_name(func),
					dlg_mod_get_tag(mod),
					dlg_func_begln(func),
					dlg_func_endln(func),
					dlg_func_descr(func),
					ELG_FUNCTION));
}

void 
dlg_register_csite(struct dlg_csite* csite)
{
  struct dlg_func* callee = dlg_prog_func(prog, dlg_csite_callee_id(csite));
  struct dlg_mod*  mod    = dlg_prog_mod(prog, dlg_csite_mod_id(csite));
  
  /* register callee if not already registered */
  if ( dlg_func_get_tag(callee) == -1 )
    dlg_register_func(callee);
            
  /* register module (i.e., file) containing call site if not already registered */      
  if ( dlg_mod_get_tag(mod) == -1 )
    dlg_mod_set_tag(mod, elg_def_file(dlg_mod_name(mod)));
  
  /* specifying the caller here might cause problems with OpenMP applications */
  dlg_csite_set_tag(csite, elg_def_csite(dlg_mod_get_tag(mod),
					 dlg_csite_line(csite),
					 dlg_func_get_tag(callee), 
					 ELG_NO_ID));
}

void 
dlg_term()
{
  dlg_exit(main_id);
  elg_cntl_msg("Application terminated");
  dlg_prog_delete(prog);  
  elg_cntl_msg("Probe module cleaned up");
}

void 
dlg_init()
{
  prog = dlg_prog_create(DLG_APPL_DESCR_FILE);
  elg_cntl_msg("Probe module initialized");
  elg_open();
  atexit(dlg_term);
}

void 
dlg_start(int func_id)
{
  main_id = func_id;
  elg_cntl_msg("Application started");
  dlg_enter(func_id);
}

void 
dlg_enter(int func_id)
{
  struct dlg_func* func = dlg_prog_func(prog, func_id);

  /* register function (i.e., region) if not already registered */
  if ( dlg_func_get_tag(func) == -1 )
#if (defined (ELG_OMPI)  || defined (ELG_OMP))
      if (omp_in_parallel())
	{
#pragma omp critical (elg_comp_dpcl_1)
	  {
	    if ( dlg_func_get_tag(func) == -1 )
	      dlg_register_func(func);
	  }
	}
      else
	{
	  dlg_register_func(func);
	}
#else
    dlg_register_func(func);
#endif

  elg_enter(dlg_func_get_tag(func));
}

void 
dlg_exit(int func_id)
{
  elg_exit();
}

void 
dlg_before(int csite_id)
{
  struct dlg_csite* csite = dlg_prog_csite(prog, csite_id);

  /* register call site if not already registered  */
  if ( dlg_csite_get_tag(csite) == -1 )
#if (defined (ELG_OMPI)  || defined (ELG_OMP))
      if (omp_in_parallel())
	{
#pragma omp critical (elg_comp_dpcl_2)
	  {
	    if ( dlg_csite_get_tag(csite) == -1 )
	      dlg_register_csite(csite);
	  }
	}
      else
	{
	  dlg_register_csite(csite);
	}
#else
    dlg_register_csite(csite);
#endif
  elg_enter_cs(dlg_csite_get_tag(csite));
}

void 
dlg_after(int csite_id)
{
  elg_exit();
}
