/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  Copyright (c) 2003-2004                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef _ELG_GEN_H
#define _ELG_GEN_H

#ifdef __cplusplus
#   define EXTERN extern "C" 
#else
#   define EXTERN extern 
#endif

#include <stdlib.h>
#include <stdio.h>

#include "elg_defs.h"

/*
 *-----------------------------------------------------------------------------
 *
 *  EPILOG Library (Event Processing, Investigating, and Logging)
 *
 *  - Low level event trace generation
 *
 *-----------------------------------------------------------------------------
 */

/*
 *-----------------------------------------------------------------------------
 * Typedefs (to make data types opaque)
 *-----------------------------------------------------------------------------
 */

/* Trace file generated at run time */
typedef struct ElgGen_struct ElgGen;

/*
 *-----------------------------------------------------------------------------
 * ElgGen
 *-----------------------------------------------------------------------------
 */

EXTERN ElgGen* ElgGen_open             ( const char* path_name, 
					 size_t buffer_size );

EXTERN void ElgGen_flush               ( ElgGen* gen );

EXTERN void ElgGen_close               ( ElgGen* gen );

EXTERN char* ElgGen_get_name           ( ElgGen* gen );

      
/* -- Writing trace records -- */

/* - Definition records - */


EXTERN void ElgGen_write_STRING        ( ElgGen* gen,
					 elg_ui4 strid,
					 elg_ui1 cntc,
					 const char* str );

EXTERN void ElgGen_write_STRING_CNT    ( ElgGen* gen,
					 const char* str );

EXTERN void ElgGen_write_MACHINE       ( ElgGen* gen,
					 elg_ui4 mid,
					 elg_ui4 nodec,
					 elg_ui4 mnid );

EXTERN void ElgGen_write_NODE          ( ElgGen* gen,
					 elg_ui4 nid,
					 elg_ui4 mid,
					 elg_ui4 cpuc,
					 elg_ui4 nnid, 
					 elg_d8  cr);

EXTERN void ElgGen_write_PROCESS       ( ElgGen* gen,
					 elg_ui4 pid,
					 elg_ui4 pnid );

EXTERN void ElgGen_write_THREAD        ( ElgGen* gen,
					 elg_ui4 tid,
					 elg_ui4 pid,
					 elg_ui4 tnid );

EXTERN void ElgGen_write_LOCATION      ( ElgGen* gen,
					 elg_ui4 lid,
					 elg_ui4 mid,
					 elg_ui4 nid,
					 elg_ui4 pid,
					 elg_ui4 tid );

EXTERN void ElgGen_write_FILE          ( ElgGen* gen,
					 elg_ui4 fid,
					 elg_ui4 fnid );

EXTERN void ElgGen_write_REGION        ( ElgGen* gen,
					 elg_ui4 rid,
					 elg_ui4 rnid,
					 elg_ui4 fid,
					 elg_ui4 begln,
					 elg_ui4 endln,
					 elg_ui4 rdid,
					 elg_ui1 rtype );

EXTERN void ElgGen_write_CALL_SITE     ( ElgGen* gen,
					 elg_ui4 csid,
					 elg_ui4 fid,
					 elg_ui4 lno,
					 elg_ui4 erid,
					 elg_ui4 lrid );

EXTERN void ElgGen_write_METRIC        ( ElgGen* gen,
					 elg_ui4 metid,
					 elg_ui4 metnid,
					 elg_ui4 metdid,
					 elg_ui1 metdtype,
					 elg_ui1 metmode,
					 elg_ui1 metiv );

EXTERN void ElgGen_write_MPI_COMM      ( ElgGen* gen,
					 elg_ui4 cid,
					 elg_ui4 grpc,
					 elg_ui1 grpv[] );


EXTERN void ElgGen_write_CART_TOPOLOGY ( ElgGen* gen,
					 elg_ui4 topid,
					 elg_ui1 ndims,
					 elg_ui4 dims[],
					 elg_ui1 periods[] );

EXTERN void ElgGen_write_CART_COORDS   ( ElgGen* gen,
					 elg_ui4 topid,
					 elg_ui4 lid,
					 elg_ui1 ndims,
					 elg_ui4 coords[] );

EXTERN void ElgGen_write_OFFSET        ( ElgGen* gen,
					 elg_d8  ltime,
                                         elg_d8  offset );

EXTERN void ElgGen_write_NUM_EVENTS    ( ElgGen* gen,
					 elg_ui4 eventc );

EXTERN void ElgGen_write_LAST_DEF      ( ElgGen* gen );



/* - Event records - */

/* Event records do not include any location identifiers. These have
   to be inserted during the merge process using
   ElgOut_localize_record(). */

/* Time stamps will be added automatically */

/* Application events */

EXTERN void ElgGen_write_ENTER         ( ElgGen* gen,
					 elg_ui4 rid,
					 elg_ui1 metc,
					 elg_ui8 metv[] );

EXTERN void ElgGen_write_ENTER_CS      ( ElgGen* gen,
					 elg_ui4 csid,
					 elg_ui1 metc,
					 elg_ui8 metv[] );

EXTERN void ElgGen_write_EXIT          ( ElgGen* gen,
					 elg_ui1 metc,
					 elg_ui8 metv[] );

/* Process ids of message sources and destinations (dpid, spid, rpid)
   are mapped onto location ids by the merging process. */

EXTERN void ElgGen_write_MPI_SEND      ( ElgGen* gen,
					 elg_ui4 dpid,
					 elg_ui4 cid,
					 elg_ui4 tag,      
					 elg_ui4 sent );

EXTERN void ElgGen_write_MPI_RECV      ( ElgGen* gen,
					 elg_ui4 spid,
					 elg_ui4 cid,
					 elg_ui4 tag );

EXTERN void ElgGen_write_MPI_COLLEXIT  ( ElgGen* gen,
					 elg_ui1 metc,
					 elg_ui8 metv[], 
					 elg_ui4 rpid,
					 elg_ui4 cid,
					 elg_ui4 sent,  
					 elg_ui4 recvd );

EXTERN void ElgGen_write_OMP_FORK      ( ElgGen* gen );

EXTERN void ElgGen_write_OMP_JOIN      ( ElgGen* gen );

EXTERN void ElgGen_write_OMP_ALOCK     ( ElgGen* gen,
					 elg_ui4 lkid );

EXTERN void ElgGen_write_OMP_RLOCK     ( ElgGen* gen,
					 elg_ui4 lkid );

EXTERN void ElgGen_write_OMP_COLLEXIT  ( ElgGen* gen,
					 elg_ui1 metc,
					 elg_ui8 metv[] );

/* Tracing events */

EXTERN void ElgGen_write_ENTER_DUMP    ( ElgGen* gen,
					 elg_ui1 metc,
					 elg_ui8 metv[] );

EXTERN void ElgGen_write_EXIT_DUMP     ( ElgGen* gen,
					 elg_ui1 metc,
					 elg_ui8 metv[] );

EXTERN void ElgGen_write_LOG_OFF       ( ElgGen* gen,
					 elg_ui1 metc,
					 elg_ui8 metv[] );

EXTERN void ElgGen_write_LOG_ON        ( ElgGen* gen,
					 elg_ui1 metc,
					 elg_ui8 metv[] );

#endif
