/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 2003-2004                                                **
**  University of Tennessee                                                **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "elg_metric.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "elg_env.h"
#include "elg_error.h"
#include "papivi.h"

#define MAX_LINE_LENGTH 8192

struct metric
{
  char* name;
  char  descr[PAPI_MAX_STR_LEN];
  int   papi_code;
};

/* EPILOG -> PAPI event map */
static struct {
  char* event_name;
  int   papi_code;
} elg2papi[] = {

  /** cache level 1 **/

  /* cache level 1 accesses */

  { "L1_ACCESS", PAPI_L1_TCA },
  { "L1_I_ACCESS", PAPI_L1_ICA },
  { "L1_D_ACCESS", PAPI_L1_DCA },
  
  /* cache level 1 reads */
  
  { "L1_READ", PAPI_L1_TCR },
  { "L1_I_READ", PAPI_L1_ICR },
  { "L1_D_READ", PAPI_L1_DCR },
  
  /* cache level 1 writes */
  
  { "L1_WRITE", PAPI_L1_TCW },
  { "L1_I_WRITE", PAPI_L1_ICW },
  { "L1_D_WRITE", PAPI_L1_DCW },
  
  /* cache level 1 hits */
  
  { "L1_HIT", PAPI_L1_TCH },
  { "L1_I_HIT", PAPI_L1_ICH },
  { "L1_D_HIT", PAPI_L1_DCH },
  
  /* cache level 1 misses */
  
  { "L1_MISS", PAPI_L1_TCM },
  { "L1_I_MISS", PAPI_L1_ICM },
  { "L1_D_MISS", PAPI_L1_DCM },
  
  /** cache level 2 **/

  /* cache level 2 accesses */

  { "L2_ACCESS", PAPI_L2_TCA },
  { "L2_I_ACCESS", PAPI_L2_ICA },
  { "L2_D_ACCESS", PAPI_L2_DCA },
  
  /* cache level 2 reads */
  
  { "L2_READ", PAPI_L2_TCR },
  { "L2_I_READ", PAPI_L2_ICR },
  { "L2_D_READ", PAPI_L2_DCR },
  
  /* cache level 2 writes */
  
  { "L2_WRITE", PAPI_L2_TCW },
  { "L2_I_WRITE", PAPI_L2_ICW },
  { "L2_D_WRITE", PAPI_L2_DCW },
  
  /* cache level 2 hits */
  
  { "L2_HIT", PAPI_L2_TCH },
  { "L2_I_HIT", PAPI_L2_ICH },
  { "L2_D_HIT", PAPI_L2_DCH },
  
  /* cache level 2 misses */
  
  { "L2_MISS", PAPI_L2_TCM },
  { "L2_I_MISS", PAPI_L2_ICM },
  { "L2_D_MISS", PAPI_L2_DCM },

  /** cache level 3 **/

  /* cache level 3 accesses */

  { "L3_ACCESS", PAPI_L3_TCA },
  { "L3_I_ACCESS", PAPI_L3_ICA },
  { "L3_D_ACCESS", PAPI_L3_DCA },
  
  /* cache level 3 reads */
  
  { "L3_READ", PAPI_L3_TCR },
  { "L3_I_READ", PAPI_L3_ICR },
  { "L3_D_READ", PAPI_L3_DCR },
  
  /* cache level 3 writes */
  
  { "L3_WRITE", PAPI_L3_TCW },
  { "L3_I_WRITE", PAPI_L3_ICW },
  { "L3_D_WRITE", PAPI_L3_DCW },
  
  /* cache level 3 hits */
  
  { "L3_HIT", PAPI_L3_TCH },
  { "L3_I_HIT", PAPI_L3_ICH },
  { "L3_D_HIT", PAPI_L3_DCH },
  
  /* cache level 3 misses */
  
  { "L3_MISS", PAPI_L3_TCM },
  { "L3_I_MISS", PAPI_L3_ICM },
  { "L3_D_MISS", PAPI_L3_DCM },
  
  /** TLB **/
  
  /* TLB misses */

  { "TLB_MISS", PAPI_TLB_TL },
  { "TLB_I_MISS", PAPI_TLB_IM },
  { "TLB_D_MISS", PAPI_TLB_DM },
  
  /** instructions **/
  
  { "INSTRUCTION", PAPI_TOT_INS },
  { "INTEGER", PAPI_INT_INS },
  { "FLOATING_POINT", PAPI_FP_INS },
  { "LOAD_STORE", PAPI_LST_INS },
  { "LOAD", PAPI_LD_INS },
  { "STORE", PAPI_SR_INS },
  { "COND_STORE", PAPI_CSR_TOT },
  { "COND_STORE_SUCCESS", PAPI_CSR_SUC },
  { "COND_STORE_UNSUCCESS", PAPI_CSR_FAL },
  { "COND_BRANCH", PAPI_BR_CN },
  { "COND_BRANCH_TAKEN", PAPI_BR_TKN },
  { "COND_BRANCH_NOTTAKEN", PAPI_BR_NTK },
  { "COND_BRANCH_PRED", PAPI_BR_PRC },
  { "COND_BRANCH_MISPRED", PAPI_BR_MSP },
  
  /** cycles **/
  
  { "CYCLES", PAPI_TOT_CYC },
  
  /** idle units **/
  
  { "INTEGER_UNIT_IDLE", PAPI_FXU_IDL },
  { "FLOAT_UNIT_IDLE", PAPI_FPU_IDL },
  { "BRANCH_UNIT_IDLE", PAPI_BRU_IDL },
  { "LOADSTORE_UNIT_IDLE", PAPI_LSU_IDL },

  /** stalls **/
  
  { "STALL_MEMORY_ACCESS", PAPI_MEM_SCY },
  { "STALL_MEMORY_READ", PAPI_MEM_RCY },
  { "STALL_MEMORY_WRITE", PAPI_MEM_WCY },
  
  /** termination of list **/

  { NULL, 0 }
}; 

struct elg_metv
{
  int EventSet;
};

/* global variables */
static struct metric* metricv[ELG_METRIC_MAXNUM];
static char   errstring[PAPI_MAX_STR_LEN];
static int    nmetrics = 0;

/* PAPI-specific error message */

void elg_metric_error(int errcode)
{
  char   errstring[PAPI_MAX_STR_LEN];

  PAPI_perror(errcode, errstring, PAPI_MAX_STR_LEN);
  elg_error_msg("%s", errstring);
}

/* test whether requested event combination valid */

void elg_metric_test()
{
  int i;
  int retval;
  int EventSet = PAPI_NULL;
  
  /* create event set */
  retval = PAPIvi_create_eventset(&EventSet);
  if ( retval != PAPI_OK)
    elg_metric_error(retval);
  
  for ( i = 0; i < nmetrics; i++ )
    {
      /* add event to event set */
      retval = PAPIvi_add_event(EventSet, metricv[i]->papi_code);
      if ( retval != PAPI_OK )
	elg_metric_error(retval);
      elg_cntl_msg("Event %s added to event set", metricv[i]->name);
    }

  retval = PAPIvi_cleanup_eventset(EventSet);
  if ( retval != PAPI_OK )
    elg_metric_error(retval);
  
  retval = PAPIvi_destroy_eventset(&EventSet);
  if ( retval != PAPI_OK )
    elg_metric_error(retval);

  elg_cntl_msg("Event set tested OK");
}

int elg_metric_open()
{
  char*  env; 
  char*  var; 
  char*  token;
  int   retval;
  int   i, j;
  PAPI_event_info_t info;
  
  nmetrics = 0;

  /* read environment variable "ELG_METRICS". Return if unset. */
  env = elg_env_metrics();
  if ( env == NULL )
    return nmetrics;
  
  var = calloc(strlen(env) + 1, sizeof(char));
  if ( var == NULL )
    elg_error();      
  strcpy(var, env);

  /* read counter names */
  token = strtok(var, ":");  
  while ( token )
    {       
      metricv[nmetrics] = malloc(sizeof(struct metric));
      if ( metricv[nmetrics] == NULL )
	elg_error();      
      
      metricv[nmetrics]->name = malloc(strlen(token) + 1);
      if ( metricv[nmetrics]->name == NULL )
	elg_error();
      memcpy(metricv[nmetrics]->name, token, strlen(token) + 1);
      nmetrics++;
      
      if ( nmetrics > ELG_METRIC_MAXNUM )
	elg_error_msg("Number of counters exceeds allowed maximum of %d", ELG_METRIC_MAXNUM);	

      token = strtok(NULL, ":");        
   }

  /* free variable var */
  free(var);
  
  if ( nmetrics == 0 )
    return nmetrics;
  
  /* initialize PAPI */
  retval = PAPIvi_library_init(PAPI_VER_CURRENT);  
  if ( retval != PAPI_VER_CURRENT )
    elg_metric_error(retval);
  
  for ( i = 0; i < nmetrics; i++ )
    {
      j = 0;
      while ( elg2papi[j].event_name != NULL )
	{
	  if ( strcmp(elg2papi[j].event_name, metricv[i]->name) == 0 )
	    {
	      metricv[i]->papi_code = elg2papi[j].papi_code;
	      break;
	    }
	  j++;
	}
      if ( elg2papi[j].event_name == NULL ) 
	elg_error_msg("Unknown counter %s", metricv[i]->name);
      
      /* get event description */
      retval = PAPIvi_get_event_info(metricv[i]->papi_code, &info);
      if ( retval != PAPI_OK )
	elg_metric_error(retval);
      strcpy(metricv[i]->descr, info.long_descr);
    }
  
  /* Check whether event combination is valid. This is done here to
     avoid errors when creating the event set for each thread, which
     would multiply the error messages. */
  elg_metric_test();
  
  return nmetrics;
}

void elg_metric_close()
{
  int i;
  
  for ( i = 0; i < nmetrics; i++ )
    {
      free (metricv[i]->name);
      free(metricv[i]);
    }
}

void elg_metric_thread_init(int (*id_fn)(void))
{
  int retval;

  if ( nmetrics == 0 )
    return;

  retval = PAPIvi_thread_init((unsigned long (*)(void))(id_fn)); 
  if ( retval != PAPI_OK)
    elg_metric_error(retval);
  elg_cntl_msg("PAPI thread support initialized");
}


int elg_metric_num()
{
  return nmetrics;
}

double elg_metric_clckrt()
{
  const PAPI_hw_info_t* hwinfo = NULL;

  hwinfo = PAPI_get_hardware_info(); 
  if ( hwinfo == NULL)
    elg_error_msg("Failed to access PAPI harware info");
  elg_cntl_msg("Clock rate: %f MHz", hwinfo->mhz);

  return hwinfo->mhz * 1000000.0;
}

const char* elg_metric_name(int i)
{
  return metricv[i]->name;
}

const char* elg_metric_descr(int i)
{
  return metricv[i]->descr;
}

int elg_metric_dtype(int i)
{
  return ELG_INTEGER;
}

int elg_metric_mode(int i)
{
  return ELG_COUNTER;
}

int elg_metric_iv(int i)
{
  return ELG_START;
}


struct elg_metv* elg_metric_create()
{
  struct elg_metv* metv;
  int retval, i;

  if ( nmetrics == 0 )
    return NULL;

  metv = malloc(sizeof(struct elg_metv));
  if ( metv == NULL )
    elg_error();
  
  /* create event set */
  metv->EventSet = PAPI_NULL;
  retval = PAPIvi_create_eventset(&metv->EventSet);
  if ( retval != PAPI_OK)
    elg_metric_error(retval);
  
  for ( i = 0; i < nmetrics; i++ )
    {
      /* add event to event set */
      retval = PAPIvi_add_event(metv->EventSet, metricv[i]->papi_code);
      if ( retval != PAPI_OK )
	elg_metric_error(retval);
    }

  retval = PAPIvi_start(metv->EventSet);
  if ( retval != PAPI_OK )
    elg_metric_error(retval);
  elg_cntl_msg("Counters started");

  return metv;
}

void elg_metric_free(struct elg_metv* metv)
{
  int retval;
  long_long papi_vals[ELG_METRIC_MAXNUM];

  if ( metv == NULL )
    return;

  retval = PAPIvi_stop(metv->EventSet, papi_vals);
  if ( retval != PAPI_OK )
    elg_metric_error(retval);

  retval = PAPIvi_cleanup_eventset(metv->EventSet);
  if ( retval != PAPI_OK )
    elg_metric_error(retval);

  retval = PAPIvi_destroy_eventset(&metv->EventSet);
  if ( retval != PAPI_OK )
    elg_metric_error(retval);
    
  free(metv);

  elg_cntl_msg("Counters stopped");
}

void elg_metric_read(struct elg_metv* metv, elg_ui8 values[])
{
  int retval;

  if ( metv == NULL )
    return;

  if (sizeof(long_long) == 8)
    {
      retval = PAPIvi_read(metv->EventSet, (long_long*) values);
    }
  else
    {
      int i;
      long_long papi_vals[ELG_METRIC_MAXNUM];
      retval = PAPIvi_read(metv->EventSet, papi_vals);
      for (i = 0; i < nmetrics; i++)
	values[i] = (elg_ui8) papi_vals[i];
    }
     
  if ( retval != PAPI_OK )
    elg_metric_error(retval);
}




