/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include "elg_rw.h"

#include <map>
#include <vector>

using namespace std;

map<vector<elg_ui4>, int> queue;
map<elg_ui4, elg_ui4> lid2pid;

void elg_verify_send(elg_ui4 spid, elg_ui4 dpid, elg_ui4 cid, elg_ui4 tag)
{
  vector<elg_ui4> key;
  key.push_back(spid); key.push_back(dpid); key.push_back(cid); key.push_back(tag);

  map<vector<elg_ui4>, int>::iterator it = queue.find(key);
  if (it == queue.end())
    queue[key] = 1;
  else
    queue[key]++;
}

void elg_verify_recv(elg_ui4 spid, elg_ui4 dpid, elg_ui4 cid, elg_ui4 tag)
{
  vector<elg_ui4> key;
  key.push_back(spid); key.push_back(dpid); key.push_back(cid); key.push_back(tag);

  map<vector<elg_ui4>, int>::iterator it = queue.find(key);
  if (it == queue.end())
    {
      fprintf(stderr, "\nInconsistent message order!\n\n");
      exit(123);
    }
  else if (queue[key] <= 0)
    {
      fprintf(stderr, "\nInconsistent message order!\n\n");
      exit(123);
    }
  else
    {
      queue[key]--;
    }
}

int main(int argc, char **argv) 
{
  ElgIn* in;
  ElgRec* rec;

  if ( (in = ElgIn_open(argv[1])) == NULL )
    {
      fprintf(stderr, "Can't open file %s\n", argv[1]);
      fflush(stderr);
      exit(EXIT_FAILURE);
    }

  while(rec = ElgIn_read_record(in))
    {
      elg_ui1 type   = ElgRec_get_type(rec);

      switch(type)
	{
	case ELG_LOCATION :
	  {
	    elg_ui4 lid, pid;

	    lid = ElgRec_read_ui4(rec);
	    ElgRec_seek(rec, 3 * sizeof(lid));
	    pid = ElgRec_read_ui4(rec);
	    lid2pid[lid] = pid;
	    break;
	  }
	case ELG_MPI_SEND :
	  {
	    printf("<SEND> ");
	    
	    elg_ui4 lid, pid, dlid, dpid, cid, tag, sent;
	    elg_d8 time;

	    lid = ElgRec_read_ui4(rec);
	    pid  = lid2pid[lid];
	    time = ElgRec_read_d8(rec); 
	    dlid = ElgRec_read_ui4(rec);
	    dpid = lid2pid[dlid];
	    cid  = ElgRec_read_ui4(rec);
	    tag  = ElgRec_read_ui4(rec);
	    sent = ElgRec_read_ui4(rec);
	    
	    printf(" lid  : %d", lid);	  
	    printf(" pid  : %d", pid);	  
	    printf(" time : %10.6f", time);	  
	    printf(" dlid : %d", dlid);	  
	    printf(" dpid : %d", dpid);	  
	    printf(" cid  : %d", cid);	  
	    printf(" tag  : %d", tag);	  
	    printf(" sent : %d", sent);	  
	    printf("\n");
      
	    elg_verify_send(pid, dpid, cid, tag);
	    
	    break;
	  }
	case ELG_MPI_RECV :
	  {
	    printf("<RECV> ");
	    	    
	    elg_ui4 lid, pid, slid, spid, cid, tag;
	    elg_d8 time;

	    lid = ElgRec_read_ui4(rec);
	    pid  = lid2pid[lid];
	    time = ElgRec_read_d8(rec); 
	    slid = ElgRec_read_ui4(rec);
	    spid = lid2pid[slid];
	    cid  = ElgRec_read_ui4(rec);
	    tag  = ElgRec_read_ui4(rec);
	    
	    printf(" lid  : %d", lid);	  
	    printf(" pid  : %d", pid);	  
	    printf(" time : %10.6f", time);	  
	    printf(" slid : %d", slid);	  
	    printf(" spid : %d", spid);	  
	    printf(" cid  : %d", cid);	  
	    printf(" tag  : %d", tag);	  
	    printf("\n");
	    
	    elg_verify_recv(spid, pid, cid, tag);
	    
	    break;
	  }
	}
      ElgRec_free(rec);
    }
  
  fprintf(stderr, "\nMessage order o.k.!\n\n");
  ElgIn_close(in);
}


















