/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/rsg.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/syssx.h>
#include <fcntl.h>
#include "elg_pform.h"

static double elg_time_base = 0.0;

/* platform specific initialization */
void elg_pform_init() {
  unsigned long long val;
  syssx(HGTIME, &val);
  elg_time_base = val - (val % 10000000000);
}

/* MPI event notification */
void elg_pform_mpi_init() {}
void elg_pform_mpi_finalize() {} 

/* directory of global file system  */
char* elg_pform_gdir() {
  return ".";
}

/* directory of local file system  */
char* elg_pform_ldir() {
  #ifdef PFORM_LDIR
    return PFORM_LDIR;
  #else
    return "/tmp";
  #endif
}

/* is a global clock provided ? */ 
int elg_pform_is_gclock() {
  return 0;
}

/* local or global wall-clock time in seconds */
double elg_pform_wtime() {
  unsigned long long val;
  syssx(HGTIME, &val);
  return (val - elg_time_base) * 1.0e-6;
}

/* platform name */
char*  elg_pform_name() {
  return "NEC-SX";
}

/* number of nodes */
int elg_pform_num_nodes() {
  return 1;
}

/* unique numeric SMP-node identifier */
long elg_pform_node_id() {
  return abs(gethostid()) %100000;
}

/* unique string SMP-node identifier */
char* elg_pform_node_name() {
  static char host_name[20];
  gethostname(host_name, 20);
  return host_name;              
}

/* number of CPUs */
int elg_pform_num_cpus() {
  rsg_info_t data;
  int id;

  id=open("/dev/rsg/own", O_RDONLY);
  ioctl(id, RSG_INFO, &data);
  close(id);
  return data.cprb.maxi_cpu;
}
