/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "pomp_lib.h"
#include "pomp_fwrapper_def.h"

#include "elg_pomp.h"
#include "elg_omplock.h"
#include "elg_ompreg.h"
#include "elg_trc.h"

/*
 *----------------------------------------------------------------
 * EPILOG implementation of the C pomp function library
 *----------------------------------------------------------------
 */
   
void POMP_Atomic_enter(struct ompregdescr* r) {
  GUARDED_ENTER(rid);
}

void POMP_Atomic_exit(struct ompregdescr* r) {
  GUARDED_EXIT();
}

void POMP_Barrier_enter(struct ompregdescr* r) {
  GUARDED_ENTER_2('b', rid, brid);
}

void POMP_Barrier_exit(struct ompregdescr* r) {
  GUARDED_COLL_EXIT();
}

void POMP_Flush_enter(struct ompregdescr* r) {
  GUARDED_ENTER(rid);
}

void POMP_Flush_exit(struct ompregdescr* r) {
  GUARDED_EXIT();
}

void POMP_Critical_begin(struct ompregdescr* r) {
  if ( pomp_tracing ) {
    struct ElgRegDescr* data = (struct ElgRegDescr*)(r->data);
    elg_omp_alock(data->brid);
    elg_enter(data->sbrid);
  }
}

void POMP_Critical_end(struct ompregdescr* r) {
  if ( pomp_tracing ) {
    struct ElgRegDescr* data = (struct ElgRegDescr*)(r->data);
    elg_exit();
    elg_omp_rlock(data->brid);
  }
}

void POMP_Critical_enter(struct ompregdescr* r) {
  GUARDED_ENTER(rid);
}

void POMP_Critical_exit(struct ompregdescr* r) {
  GUARDED_EXIT();
}

void POMP_For_enter(struct ompregdescr* r) {
  GUARDED_ENTER_2('p', sbrid, rid);
}

void POMP_For_exit(struct ompregdescr* r) {
  GUARDED_COLL_EXIT();
}

void POMP_Master_begin(struct ompregdescr* r) {
  GUARDED_ENTER(rid);
}

void POMP_Master_end(struct ompregdescr* r) {
  GUARDED_EXIT();
}

void POMP_Parallel_begin(struct ompregdescr* r) {
  GUARDED_ENTER(rid);
}

void POMP_Parallel_end(struct ompregdescr* r) {
  GUARDED_COLL_EXIT();
}

void POMP_Parallel_fork(struct ompregdescr* r) {
  if ( pomp_tracing ) {
    elg_omp_fork();
  }
}

void POMP_Parallel_join(struct ompregdescr* r) {
  if ( pomp_tracing ) {
    elg_omp_join();
  }
}

void POMP_Section_begin(struct ompregdescr* r) {
  GUARDED_ENTER(sbrid);
}

void POMP_Section_end(struct ompregdescr* r) {
  GUARDED_EXIT();
}

void POMP_Sections_enter(struct ompregdescr* r) {
  GUARDED_ENTER_2('p', sbrid, rid);
}

void POMP_Sections_exit(struct ompregdescr* r) {
  GUARDED_COLL_EXIT();
}

void POMP_Single_begin(struct ompregdescr* r) {
  GUARDED_ENTER(sbrid);
}

void POMP_Single_end(struct ompregdescr* r) {
  GUARDED_EXIT();
}

void POMP_Single_enter(struct ompregdescr* r) {
  GUARDED_ENTER(rid);
}

void POMP_Single_exit(struct ompregdescr* r) {
  GUARDED_COLL_EXIT();
}

void POMP_Workshare_enter(struct ompregdescr* r) {
  GUARDED_ENTER_2('p', sbrid, rid);
}

void POMP_Workshare_exit(struct ompregdescr* r) {
  GUARDED_COLL_EXIT();
}

/*
 *----------------------------------------------------------------
 * C Wrapper for OpenMP API
 *----------------------------------------------------------------
 */

void POMP_Init_lock(omp_lock_t *s) {
  if ( pomp_tracing ) {
    elg_enter(elg_omp_regid[ELG__OMP_INIT_LOCK]);
    omp_init_lock(s);
    elg_lock_init(s);
    elg_exit();
  } else {
    omp_init_lock(s);
    elg_lock_init(s);
  }
}

void POMP_Destroy_lock(omp_lock_t *s) {
  if ( pomp_tracing ) {
    elg_enter(elg_omp_regid[ELG__OMP_DESTROY_LOCK]);
    omp_destroy_lock(s);
    elg_lock_destroy(s);
    elg_exit();
  } else {
    omp_destroy_lock(s);
    elg_lock_destroy(s);
  }
}

void POMP_Set_lock(omp_lock_t *s) {
  if ( pomp_tracing ) {
    elg_enter(elg_omp_regid[ELG__OMP_SET_LOCK]);
    omp_set_lock(s);
    elg_omp_alock(elg_lock_id(s));
    elg_exit();
  } else {
    omp_set_lock(s);
  }
}

void POMP_Unset_lock(omp_lock_t *s) {
  if ( pomp_tracing ) {
    elg_enter(elg_omp_regid[ELG__OMP_UNSET_LOCK]);
    omp_unset_lock(s);
    elg_omp_rlock(elg_lock_id(s));
    elg_exit();
  } else {
    omp_unset_lock(s);
  }
}

int  POMP_Test_lock(omp_lock_t *s) {
  if ( pomp_tracing ) {
    int result;

    elg_enter(elg_omp_regid[ELG__OMP_TEST_LOCK]);
    result = omp_test_lock(s);
    if (result) elg_omp_alock(elg_lock_id(s));
    elg_exit();
    return result;     
  } else {
    return omp_test_lock(s);
  }
}

void POMP_Init_nest_lock(omp_nest_lock_t *s) {
  if ( pomp_tracing ) {
    elg_enter(elg_omp_regid[ELG__OMP_INIT_NEST_LOCK]);
    omp_init_nest_lock(s);
    elg_lock_init(s);
    elg_exit();
  } else {
    omp_init_nest_lock(s);
    elg_lock_init(s);
  }
}

void POMP_Destroy_nest_lock(omp_nest_lock_t *s) {
  if ( pomp_tracing ) {
    elg_enter(elg_omp_regid[ELG__OMP_DESTROY_NEST_LOCK]);
    omp_destroy_nest_lock(s);
    elg_lock_destroy(s);
    elg_exit();
  } else {
    omp_destroy_nest_lock(s);
    elg_lock_destroy(s);
  }
}

void POMP_Set_nest_lock(omp_nest_lock_t *s) {
  if ( pomp_tracing ) {
    elg_enter(elg_omp_regid[ELG__OMP_SET_NEST_LOCK]);
    omp_set_nest_lock(s);
    elg_omp_alock(elg_lock_id(s));
    elg_exit();
  } else {
    omp_set_nest_lock(s);
  }
}

void POMP_Unset_nest_lock(omp_nest_lock_t *s) {
  if ( pomp_tracing ) {
    elg_enter(elg_omp_regid[ELG__OMP_UNSET_NEST_LOCK]);
    omp_unset_nest_lock(s);
    elg_omp_rlock(elg_lock_id(s));
    elg_exit();
  } else {
    omp_unset_nest_lock(s);
  }
}

int  POMP_Test_nest_lock(omp_nest_lock_t *s) {
  if ( pomp_tracing ) {
    int result;

    elg_enter(elg_omp_regid[ELG__OMP_TEST_NEST_LOCK]);
    result = omp_test_nest_lock(s);
    if (result) elg_omp_alock(elg_lock_id(s));
    elg_exit();
    return result;
  } else {
    return omp_test_nest_lock(s);
  }
}

/*
 *----------------------------------------------------------------
 * Fortran Wrapper for OpenMP API
 *----------------------------------------------------------------
 */

#if defined(__ICC) || defined(__ECC) || defined(_SX)
  #define CALLFSUB(name) name
#else
  #define CALLFSUB(name) FSUB(name)
#endif

void FSUB(POMP_Init_lock)(omp_lock_t *s) {
  if ( pomp_tracing ) {
    elg_enter(elg_omp_regid[ELG__OMP_INIT_LOCK]);
    CALLFSUB(omp_init_lock)(s);
    elg_lock_init(s);
    elg_exit();
  } else {
    CALLFSUB(omp_init_lock)(s);
    elg_lock_init(s);
  }
}

void FSUB(POMP_Destroy_lock)(omp_lock_t *s) {
  if ( pomp_tracing ) {
    elg_enter(elg_omp_regid[ELG__OMP_DESTROY_LOCK]);
    CALLFSUB(omp_destroy_lock)(s);
    elg_lock_destroy(s);
    elg_exit();
  } else {
    CALLFSUB(omp_destroy_lock)(s);
    elg_lock_destroy(s);
  }
}

void FSUB(POMP_Set_lock)(omp_lock_t *s) {
  if ( pomp_tracing ) {
    elg_enter(elg_omp_regid[ELG__OMP_SET_LOCK]);
    CALLFSUB(omp_set_lock)(s);
    elg_omp_alock(elg_lock_id(s));
    elg_exit();
  } else {
    CALLFSUB(omp_set_lock)(s);
  }
}

void FSUB(POMP_Unset_lock)(omp_lock_t *s) {
  if ( pomp_tracing ) {
    elg_enter(elg_omp_regid[ELG__OMP_UNSET_LOCK]);
    CALLFSUB(omp_unset_lock)(s);
    elg_omp_rlock(elg_lock_id(s));
    elg_exit();
  } else {
    CALLFSUB(omp_unset_lock)(s);
  }
}

int  FSUB(POMP_Test_lock)(omp_lock_t *s) {
  if ( pomp_tracing ) {
    int result;

    elg_enter(elg_omp_regid[ELG__OMP_TEST_LOCK]);
    result = CALLFSUB(omp_test_lock)(s);
    if (result) elg_omp_alock(elg_lock_id(s));
    elg_exit();
    return result;     
  } else {
    return CALLFSUB(omp_test_lock)(s);
  }
}

#ifndef __osf__
void FSUB(POMP_Init_nest_lock)(omp_nest_lock_t *s) {
  if ( pomp_tracing ) {
    elg_enter(elg_omp_regid[ELG__OMP_INIT_NEST_LOCK]);
    CALLFSUB(omp_init_nest_lock)(s);
    elg_lock_init(s);
    elg_exit();
  } else {
    CALLFSUB(omp_init_nest_lock)(s);
    elg_lock_init(s);
  }
}

void FSUB(POMP_Destroy_nest_lock)(omp_nest_lock_t *s) {
  if ( pomp_tracing ) {
    elg_enter(elg_omp_regid[ELG__OMP_DESTROY_NEST_LOCK]);
    CALLFSUB(omp_destroy_nest_lock)(s);
    elg_lock_destroy(s);
    elg_exit();
  } else {
    CALLFSUB(omp_destroy_nest_lock)(s);
    elg_lock_destroy(s);
  }
}

void FSUB(POMP_Set_nest_lock)(omp_nest_lock_t *s) {
  if ( pomp_tracing ) {
    elg_enter(elg_omp_regid[ELG__OMP_SET_NEST_LOCK]);
    CALLFSUB(omp_set_nest_lock)(s);
    elg_omp_alock(elg_lock_id(s));
    elg_exit();
  } else {
    CALLFSUB(omp_set_nest_lock)(s);
  }
}

void FSUB(POMP_Unset_nest_lock)(omp_nest_lock_t *s) {
  if ( pomp_tracing ) {
    elg_enter(elg_omp_regid[ELG__OMP_UNSET_NEST_LOCK]);
    CALLFSUB(omp_unset_nest_lock)(s);
    elg_omp_rlock(elg_lock_id(s));
    elg_exit();
  } else {
    CALLFSUB(omp_unset_nest_lock)(s);
  }
}

int  FSUB(POMP_Test_nest_lock)(omp_nest_lock_t *s) {
  if ( pomp_tracing ) {
    int result;

    elg_enter(elg_omp_regid[ELG__OMP_TEST_NEST_LOCK]);
    result = CALLFSUB(omp_test_nest_lock)(s);
    if (result) elg_omp_alock(elg_lock_id(s));
    elg_exit();
    return result;
  } else {
    return CALLFSUB(omp_test_nest_lock)(s);
  }
}
#endif
