/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (C) 1998-2003                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/


#ifndef ELG_POMP2_H
#define ELG_POMP2_H

#include "opari_omp.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Datatypes
 */
typedef void* POMP_Handle_t;
typedef int p_int32;
typedef long p_int64;

enum pomp_group_t { GRP_PARALLEL=0, GRP_LOOP=1, GRP_WSHARE=2, GRP_SYNC=3,
  GRP_USER=4, GRP_RUNTIME=5, GRP_UNKNOWN=-1 };

enum pomp_grpval_t { GV_NONE=0, GV_EVENT=1, GV_ENTEREXIT=2, GV_BEGINEND=3,
  GV_ITER_CHUNK=4, GV_ITER_EVENT=5, GV_ITER_BEGINEND=6, GV_UNKNOWN=-1 };


/*
 * Control Routines
 */
extern p_int32 POMP_Init(void);
extern void POMP_Finalize(void);
extern void POMP_On(void);
extern void POMP_Off(void);
extern void POMP_Set_group(enum pomp_group_t group, enum pomp_grpval_t level);
extern p_int32 POMP_Get_handle(POMP_Handle_t* handle, char ctc[]);

/*
 * Event Routines Group PARALLEL
 */
extern p_int32 POMP_Parallel_enter(POMP_Handle_t* handle, p_int32 thread_id,
               p_int32 num_threads, p_int32 if_expr_result, char ctc[]);
extern p_int32 POMP_Parallel_begin(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Parallel_end(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Parallel_exit(POMP_Handle_t* handle, p_int32 thread_id);

/*
 * Event Routines Group LOOP
 */
extern p_int32 POMP_Loop_enter(POMP_Handle_t* handle,
               p_int32 thread_id, p_int64 chunk_size, p_int64 init_iter,
               p_int64 final_iter, p_int64 incr, char ctc[]);
extern p_int32 POMP_Loop_chunk_begin(POMP_Handle_t* handle,
               p_int32 thread_id, p_int64 init_iter, p_int64 final_iter);
extern p_int32 POMP_Loop_iter_begin(POMP_Handle_t* handle,
               p_int32 thread_id, p_int64 iter);
extern p_int32 POMP_Loop_iter_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int64 iter);
extern p_int32 POMP_Loop_iter_end(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Loop_chunk_end(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Loop_exit(POMP_Handle_t* handle, p_int32 thread_id);

/*
 * Event Routines Group WORKSHARE
 */
extern p_int32 POMP_Workshare_enter(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]);
extern p_int32 POMP_Workshare_begin(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Workshare_end(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Workshare_exit(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Sections_enter(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]);
extern p_int32 POMP_Section_begin(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 section_num);
extern p_int32 POMP_Section_end(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Sections_exit(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Single_enter(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]);
extern p_int32 POMP_Single_begin(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Single_end(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Single_exit(POMP_Handle_t* handle, p_int32 thread_id);

/*
 * Event Routines Group SYNC
 */
extern p_int32 POMP_Critical_enter(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]);
extern p_int32 POMP_Critical_begin(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Critical_end(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Critical_exit(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Master_begin(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]);
extern p_int32 POMP_Master_end(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Barrier_enter(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]);
extern p_int32 POMP_Barrier_exit(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Implicit_barrier_enter(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]);
extern p_int32 POMP_Implicit_barrier_exit(POMP_Handle_t* handle,
               p_int32 thread_id);
extern p_int32 POMP_Flush_event(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]);
extern p_int32 POMP_Atomic_event(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]);
extern p_int32 POMP_Ordered_enter(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]);
extern p_int32 POMP_Ordered_begin(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Ordered_end(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Ordered_exit(POMP_Handle_t* handle, p_int32 thread_id);

/*
 * Event Routines Group USER
 */
extern p_int32 POMP_Function_enter(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]);
extern p_int32 POMP_Function_begin(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]);
extern p_int32 POMP_Function_end(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Function_exit(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_Function_event(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]);
extern p_int32 POMP_User_region_begin(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]);
extern p_int32 POMP_User_region_end(POMP_Handle_t* handle, p_int32 thread_id);
extern p_int32 POMP_User_event(POMP_Handle_t* handle,
               p_int32 thread_id, char ctc[]);

/*
 * Event Routines Group RUNTIME
 */
extern p_int32 POMP_Init_lock_event(POMP_Handle_t* handle,
               p_int32 thread_id, omp_lock_t* lock_id, char ctc[]);
extern p_int32 POMP_Destroy_lock_event(POMP_Handle_t* handle,
               p_int32 thread_id, omp_lock_t* lock_id, char ctc[]);
extern p_int32 POMP_Set_lock_enter(POMP_Handle_t* handle,
               p_int32 thread_id, omp_lock_t* lock_id, char ctc[]);
extern p_int32 POMP_Set_lock_exit(POMP_Handle_t* handle,
               p_int32 thread_id, omp_lock_t* lock_id);
extern p_int32 POMP_Set_lock_event(POMP_Handle_t* handle,
               p_int32 thread_id, omp_lock_t* lock_id, char ctc[]);
extern p_int32 POMP_Unset_lock_event(POMP_Handle_t* handle,
               p_int32 thread_id, omp_lock_t* lock_id, char ctc[]);
extern p_int32 POMP_Test_lock_event(POMP_Handle_t* handle,
               p_int32 thread_id, omp_lock_t* lock_id, p_int32 was_lock_set,
               char ctc[]);
extern p_int32 POMP_Init_nest_lock_event(POMP_Handle_t* handle,
               p_int32 thread_id, omp_nest_lock_t* lock_id, char ctc[]);
extern p_int32 POMP_Destroy_nest_lock_event(POMP_Handle_t* handle,
               p_int32 thread_id, omp_nest_lock_t* lock_id, char ctc[]);
extern p_int32 POMP_Set_nest_lock_enter(POMP_Handle_t* handle,
               p_int32 thread_id, omp_nest_lock_t* lock_id, char ctc[]);
extern p_int32 POMP_Set_nest_lock_exit(POMP_Handle_t* handle,
               p_int32 thread_id, omp_nest_lock_t* lock_id);
extern p_int32 POMP_Set_nest_lock_event(POMP_Handle_t* handle,
               p_int32 thread_id, omp_nest_lock_t* lock_id, char ctc[]);
extern p_int32 POMP_Unset_nest_lock_event(POMP_Handle_t* handle,
               p_int32 thread_id, omp_nest_lock_t* lock_id, char ctc[]);
extern p_int32 POMP_Test_nest_lock_event(POMP_Handle_t* handle,
               p_int32 thread_id, omp_nest_lock_t* lock_id,
               p_int32 was_lock_set, char ctc[]);
extern p_int32 POMP_Set_num_threads_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 num_threads, char ctc[]); 
extern p_int32 POMP_Get_num_threads_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 num_threads, char ctc[]); 
extern p_int32 POMP_Get_max_threads_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 num_threads, char ctc[]);
extern p_int32 POMP_Get_thread_num_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 thread_num, char ctc[]); 
extern p_int32 POMP_Get_num_procs_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 num_procs, char ctc[]); 
extern p_int32 POMP_In_parallel_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 is_parallel, char ctc[]); 
extern p_int32 POMP_Set_dynamic_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 is_dynamic, char ctc[]); 
extern p_int32 POMP_Get_dynamic_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 is_dynamic, char ctc[]);
extern p_int32 POMP_Set_nested_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 is_nested, char ctc[]); 
extern p_int32 POMP_Get_nested_event(POMP_Handle_t* handle,
               p_int32 thread_id, p_int32 is_nested, char ctc[]);

#ifdef __cplusplus
}
#endif

#endif
