/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  Copyright (c) 2003-2004                                                **
**  University of Tennessee                                                **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef _ELG_READCB_H
#define _ELG_READCB_H

#ifdef __cplusplus
#   define EXTERN extern "C" 
#else
#   define EXTERN extern 
#endif

#include "elg_defs.h"


/*
 *-----------------------------------------------------------------------------
 *
 *  EPILOG Library (Event Processing, Investigating, and Logging)
 *
 *  - Interface for reading EPILOG trace records from a file.  
 *
 *-----------------------------------------------------------------------------
 */

typedef struct ElgRCB_struct ElgRCB;

/* Open and close trace file */
EXTERN ElgRCB* elg_read_open ( const char* path );
EXTERN void elg_read_close ( ElgRCB* handle );

/* Activate / deactivate the callback for specified event type
   Use constant ELG_ALL_EVENTS for processing all event types 
   By default all events are activated */
EXTERN void elg_read_activate( ElgRCB* handle, elg_ui1 type );
EXTERN void elg_read_deactivate( ElgRCB* handle, elg_ui1 type );

/* Read next record: This function will call the functions defined below
   Two functions are called for every record: elg_read_ALL() and the
   specific function matching the record's type if this type is activated.
   elg_read_next() returns 1 after successfully reading a trace record, 0
   else. */
EXTERN int  elg_read_next ( ElgRCB* handle, void* userdata );

/*
 *-----------------------------------------------------------------------------
 *
 *  EPILOG Library (Event Processing, Investigating, and Logging)
 *
 *  - Callback functions for read interface
 *
 *-----------------------------------------------------------------------------
 */

/* Two functions are called for every record: elg_read_ALL() and the
   specific function matching the record's type. */

/** - All records - **/

EXTERN void elg_readcb_ALL           ( elg_ui1 type, 
				       elg_ui1 length,
                                       void* userdata );

/** - Definition records - **/


EXTERN void elg_readcb_STRING        ( elg_ui4 strid,
				       elg_ui1 cntc,
				       const char* str,
                                       void* userdata );

EXTERN void elg_readcb_STRING_CNT    ( const char* str,
                                       void* userdata );

EXTERN void elg_readcb_MACHINE       ( elg_ui4 mid,
				       elg_ui4 nodec,
				       elg_ui4 mnid,
                                       void* userdata );

EXTERN void elg_readcb_NODE          ( elg_ui4 nid,
				       elg_ui4 mid,
				       elg_ui4 cpuc,
				       elg_ui4 nnid, 
				       elg_d8  cr,
                                       void* userdata );

EXTERN void elg_readcb_PROCESS       ( elg_ui4 pid,
				       elg_ui4 pnid,
                                       void* userdata );

EXTERN void elg_readcb_THREAD        ( elg_ui4 tid,
				       elg_ui4 pid,
				       elg_ui4 tnid,
                                       void* userdata );

EXTERN void elg_readcb_LOCATION      ( elg_ui4 lid,
				       elg_ui4 mid,
				       elg_ui4 nid,
				       elg_ui4 pid,
				       elg_ui4 tid,
                                       void* userdata );

EXTERN void elg_readcb_FILE          ( elg_ui4 fid,
				       elg_ui4 fnid,
                                       void* userdata );

EXTERN void elg_readcb_REGION        ( elg_ui4 rid,
				       elg_ui4 rnid,
				       elg_ui4 fid,
				       elg_ui4 begln,
				       elg_ui4 endln,
				       elg_ui4 rdid,
				       elg_ui1 rtype,
                                       void* userdata );

EXTERN void elg_readcb_CALL_SITE     ( elg_ui4 csid,
				       elg_ui4 fid,
				       elg_ui4 lno,
				       elg_ui4 erid,
				       elg_ui4 lrid,
                                       void* userdata );

EXTERN void elg_readcb_METRIC        ( elg_ui4 metid,
				       elg_ui4 metnid,
				       elg_ui4 metdid,
				       elg_ui1 metdtype,
				       elg_ui1 metmode,
				       elg_ui1 metiv,
                                       void* userdata );

EXTERN void elg_readcb_MPI_COMM      ( elg_ui4 cid,
				       elg_ui4 grpc,
				       elg_ui1 grpv[],
                                       void* userdata );

EXTERN void elg_readcb_CART_TOPOLOGY ( elg_ui4 topid,
				       elg_ui1 ndims,
				       elg_ui4 dimv[],
				       elg_ui1 periodv[],
				       void* userdata );

EXTERN void elg_readcb_CART_COORDS   ( elg_ui4 topid,
				       elg_ui4 lid,
				       elg_ui1 ndims,
				       elg_ui4 coordv[],
				       void* userdata);

EXTERN void elg_readcb_OFFSET        ( elg_d8  ltime,
				       elg_d8  offset,
                                       void* userdata );

EXTERN void elg_readcb_NUM_EVENTS    ( elg_ui4 eventc,
                                       void* userdata );

EXTERN void elg_readcb_LAST_DEF      ( void* userdata );



/** - Event records - **/

/* Application events */


EXTERN void elg_readcb_ENTER         ( elg_ui4 lid, 
				       elg_d8  time,
				       elg_ui4 rid,
				       elg_ui1 metc,
				       elg_ui8 metv[],
                                       void* userdata );

EXTERN void elg_readcb_ENTER_CS      ( elg_ui4 lid, 
				       elg_d8  time,
				       elg_ui4 csid,
				       elg_ui1 metc,
				       elg_ui8 metv[],
                                       void* userdata );

EXTERN void elg_readcb_EXIT          ( elg_ui4 lid, 
				       elg_d8  time,
				       elg_ui1 metc,
				       elg_ui8 metv[],
                                       void* userdata );

EXTERN void elg_readcb_MPI_SEND      ( elg_ui4 lid, 
				       elg_d8  time,
				       elg_ui4 dlid,
				       elg_ui4 cid,
				       elg_ui4 tag,      
				       elg_ui4 sent,
                                       void* userdata );

EXTERN void elg_readcb_MPI_RECV      ( elg_ui4 lid, 
				       elg_d8  time,
				       elg_ui4 slid,
				       elg_ui4 cid,
				       elg_ui4 tag,
                                       void* userdata );

EXTERN void elg_readcb_MPI_COLLEXIT  ( elg_ui4 lid, 
				       elg_d8  time,
				       elg_ui1 metc,
				       elg_ui8 metv[], 
				       elg_ui4 rlid,
				       elg_ui4 cid,
				       elg_ui4 sent,  
				       elg_ui4 recvd,
                                       void* userdata );

EXTERN void elg_readcb_OMP_FORK      ( elg_ui4 lid, 
				       elg_d8  time,
                                       void* userdata );

EXTERN void elg_readcb_OMP_JOIN      ( elg_ui4 lid, 
				       elg_d8  time,
                                       void* userdata );

EXTERN void elg_readcb_OMP_ALOCK     ( elg_ui4 lid, 
				       elg_d8  time,
				       elg_ui4 lkid,
                                       void* userdata );

EXTERN void elg_readcb_OMP_RLOCK     ( elg_ui4 lid, 
				       elg_d8  time,
				       elg_ui4 lkid,
                                       void* userdata );

EXTERN void elg_readcb_OMP_COLLEXIT  ( elg_ui4 lid, 
				       elg_d8  time,
				       elg_ui1 metc,
				       elg_ui8 metv[],
                                       void* userdata );

/* Tracing events */

EXTERN void elg_readcb_ENTER_DUMP    ( elg_ui4 lid, 
				       elg_d8  time,
				       elg_ui1 metc,
				       elg_ui8 metv[],
                                       void* userdata );

EXTERN void elg_readcb_EXIT_DUMP     ( elg_ui4 lid, 
				       elg_d8  time,
				       elg_ui1 metc,
				       elg_ui8 metv[],
                                       void* userdata );

EXTERN void elg_readcb_LOG_OFF       ( elg_ui4 lid, 
				       elg_d8  time,
				       elg_ui1 metc,
				       elg_ui8 metv[],
                                       void* userdata );

EXTERN void elg_readcb_LOG_ON        ( elg_ui4 lid, 
				       elg_d8  time,
				       elg_ui1 metc,
				       elg_ui8 metv[],
                                       void* userdata );
			
#endif
