/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (c) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  Copyright (c) 2003-2004                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#ifndef _ELG_RW_H
#define _ELG_RW_H

#ifdef __cplusplus
#   define EXTERN extern "C" 
#else
#   define EXTERN extern 
#endif

#include <stdlib.h>
#include <stdio.h>

#include "elg_defs.h"

/*
 *-----------------------------------------------------------------------------
 *
 *  EPILOG Library (Event Processing, Investigating, and Logging)
 *
 *  - Low level event trace reading / (re)writing
 *
 *-----------------------------------------------------------------------------
 */

/*
 *-----------------------------------------------------------------------------
 * Typedefs (to make data types opaque)
 *-----------------------------------------------------------------------------
 */


/* Trace file for reading */
typedef struct ElgIn_struct ElgIn;

/* Trace file for writing resp. merging */
typedef struct ElgOut_struct ElgOut;

/* Trace record */
typedef struct ElgRec_struct ElgRec;


/*
 *-----------------------------------------------------------------------------
 * ElgRec
 *----------------------------------------------------------------------------- 
*/

/* Create records (list is not complete yet) */

EXTERN ElgRec* ElgRec_create_empty( elg_ui1 length,
                                    elg_ui1 type );

EXTERN ElgRec* ElgRec_create_NODE       ( elg_ui4 nid,
					  elg_ui4 mid,
					  elg_ui4 cpuc,
					  elg_ui4 nnid, 
					  elg_d8  cr );
EXTERN ElgRec* ElgRec_create_NUM_EVENTS ( elg_ui4 eventc );
EXTERN ElgRec* ElgRec_create_LAST_DEF   ();

/* Free record */

EXTERN void  ElgRec_free                ( ElgRec* rec );

/* Reading record header */

EXTERN elg_ui1 ElgRec_get_type          ( ElgRec* rec ); 
EXTERN elg_ui1 ElgRec_get_length        ( ElgRec* rec ); 

/* Predicate on record type */

EXTERN int ElgRec_is_event              ( ElgRec* rec ); 

/* Reading record data -*/

EXTERN elg_ui1 ElgRec_read_ui1          ( ElgRec* rec ); 
EXTERN elg_ui4 ElgRec_read_ui4          ( ElgRec* rec ); 
EXTERN elg_ui8 ElgRec_read_ui8          ( ElgRec* rec ); 
EXTERN elg_d8  ElgRec_read_d8           ( ElgRec* rec );
EXTERN char*   ElgRec_read_string       ( ElgRec* rec );

/* Set read position */

EXTERN int     ElgRec_seek              ( ElgRec* rec,
					  size_t offset );
/* Writing record data -*/

EXTERN ElgRec* ElgRec_write_ui1         ( ElgRec* rec,
					 elg_ui1 ui1,
					 size_t offset ); 
EXTERN ElgRec* ElgRec_write_ui4         ( ElgRec* rec,
					  elg_ui4 ui4,
					  size_t offset ); 
EXTERN ElgRec* ElgRec_write_ui8         ( ElgRec* rec,
					  elg_ui8 ui8,
					  size_t offset ); 
EXTERN ElgRec* ElgRec_write_d8          ( ElgRec* rec,
					  elg_d8 d8,
					  size_t offset );
/*
 *-----------------------------------------------------------------------------
 * ElgIn
 *-----------------------------------------------------------------------------
 */

/* Open and close trace file */

EXTERN ElgIn*  ElgIn_open               ( const char* path_name );
EXTERN int     ElgIn_close              ( ElgIn* in );

/* EPILOG version */

EXTERN double  ElgIn_get_version        (ElgIn* in);

/* File position and error management */

EXTERN int ElgIn_setpos                 ( ElgIn* in, 
					  const fpos_t* pos );
EXTERN int ElgIn_getpos                 ( ElgIn* in, 
					  fpos_t* pos );
EXTERN int ElgIn_rewind                 ( ElgIn* in );

EXTERN int ElgIn_ferror                 ( ElgIn* in );
EXTERN int ElgIn_feof                   ( ElgIn* in );

/* Reading trace records */

EXTERN ElgRec* ElgIn_read_record        ( ElgIn* in ); 
EXTERN int     ElgIn_skip_record        ( ElgIn* in );

/*
 *-----------------------------------------------------------------------------
 * ElgOut
 *-----------------------------------------------------------------------------
 */

/* Open and close trace file */

EXTERN ElgOut* ElgOut_open              ( const char* path_name );
EXTERN int     ElgOut_close             ( ElgOut* out );


/* Write records */

/* NOTE: no byte-order check */

EXTERN int   ElgOut_localize_record     ( ElgOut* out, 
					  ElgRec* rec,
					  elg_ui4 lid);
EXTERN int   ElgOut_write_record        ( ElgOut* out, 
					  ElgRec* rec );
#endif
