/****************************************************************************
**  KOJAK                                                                  **
**  http://www.fz-juelich.de/zam/kojak/                                    **
*****************************************************************************
**  Copyright (C) 1998-2004                                                **
**  Forschungszentrum Juelich, Zentralinstitut fuer Angewandte Mathematik  **
**                                                                         **
**  Copyright (c) 2003-2004                                                **
**  University of Tennessee, Innovative Computing Laboratory               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/

#include <cmath>
#include <cstdlib>
#include <earl.h>
#include <iostream>
#include <sys/time.h>

#include "Analyzer.h"

using namespace earl;
using namespace std;

int main(int argc, char* argv[])
{
  if ( argc != 2 )
    {
      cerr << "Wrong # arguments." << endl;
      cerr << "Usage: " << argv[0] << " <filename>" << endl; 
      exit(EXIT_FAILURE);
    }
  try 
    {
      struct timeval start, end;
      long seconds;

      gettimeofday(&start, 0);

      // open trace file
      EventTrace trace(argv[1]);
      cerr << "Total number of events: " << trace.get_nevents() << endl;

      // start analysis
      Analyzer analyzer(&trace);
      analyzer.run();

      // calculate time needed by analysis
      gettimeofday(&end, 0);
      seconds = long(rint((end.tv_usec * 1.0e-6) + end.tv_sec) - ((start.tv_usec * 1.0e-6) + start.tv_sec));
      cerr << "Elapsed time (h:m:s): ";
      cerr << seconds / 3600 << " : " 
	   << (seconds % 3600) / 60 << " : "
	   << seconds % 60
	   << endl;      

      // calculate event rate
      if ( seconds == 0 )
	seconds = 1;
      cerr << "Events per second: " << trace.get_nevents() / seconds << endl;
    }
  catch ( earl::Error error ) 
    {
      cerr << error.get_msg() << "\n";
      exit(EXIT_FAILURE);
    } 
} 


